//
//  DownloadViewController.m
//  TGXF
//
//  Created by James Hudson on 28/02/2014.
//  Copyright (c) 2014 James Hudson. All rights reserved.
//

#import "DownloadViewController.h"

#include "tgxf-receive.h"

@interface DownloadViewController ()
@property (strong, nonatomic) IBOutlet UILabel *frameLabel;
@property (strong, nonatomic) IBOutlet UIProgressView *progressBar;
@property (strong, nonatomic) IBOutlet UILabel *progressLabel;
@property (strong, nonatomic) IBOutlet UILabel *errorsLabel;
@property (strong, nonatomic) IBOutlet UIButton *playPauseButton;

@end

@implementation DownloadViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
    
    self.readerView.readerDelegate = self;
    
    [self.readerView setTorchMode: 0];
    
    UIApplication *app = [UIApplication sharedApplication];
    [self.readerView willRotateToInterfaceOrientation: app.statusBarOrientation
                                        duration: 0];
    
    if(TARGET_IPHONE_SIMULATOR) {
        cameraSim = [[ZBarCameraSimulator alloc]
                     initWithViewController: self];
        cameraSim.readerView = self.readerView;
    }
}

- (void) viewWillAppear:(BOOL)animated {
    [self startReading];
    [UIApplication sharedApplication].idleTimerDisabled = YES;
}

- (void) viewDidAppear:(BOOL)animated {

}


- (void) viewWillDisappear:(BOOL)animated {
    [self.readerView stop];
    [UIApplication sharedApplication].idleTimerDisabled = NO;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) startReading {
    
    // Nuke the temporary file
    NSString * filename = [NSString stringWithFormat:@"%@/image.jpg", [self applicationDocumentsDirectory]];
    
    NSError *error;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    [fileManager removeItemAtPath:filename error:&error];
    if (error){
        NSLog(@"%@", error);
    }
    
    tgxf_reset();
    [self.readerView start];
    self.playPauseButton.titleLabel.text = @"Recording";
}

- (IBAction)playPauseToggled:(id)sender {
    if (self.readerView.isHidden) {
        [self startReading];
    }
    else {
        [self.readerView stop];
        self.playPauseButton.titleLabel.text = @"Stopped";
    }
}

- (NSString *)applicationDocumentsDirectory {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
    return basePath;
}


- (void) readerView: (ZBarReaderView*) view
     didReadSymbols: (ZBarSymbolSet*) syms
          fromImage: (UIImage*) img
{
    NSString * filename = [NSString stringWithFormat:@"%@/image.jpg", [self applicationDocumentsDirectory]];
    
    // do something useful with results
   for(ZBarSymbol *sym in syms) {
       enum TgxfReceiveState ret = decode_tgxf_frame(sym.zbarSymbol, [filename UTF8String]);
       
       if (ret == TGXF_RECEIVE_STATE_FAILED) {
            self.progressLabel.text = @"Transfer failed";
            return;
       }
       
       if (ret != TGXF_RECEIVE_STATE_IN_PROGRESS) {
           UIImage *testImage = [UIImage imageWithContentsOfFile:filename];
           
           if (!testImage || testImage.size.width == 0) {
               // Must be a text file
               NSString * filename = [NSString stringWithFormat:@"%@/image.jpg", [self applicationDocumentsDirectory]];
               NSString *text = [NSString stringWithContentsOfFile:filename encoding:NSUTF8StringEncoding error:nil];
               if (text) {
                   UIPasteboard *pasteboard = [UIPasteboard generalPasteboard];
                   pasteboard.string = text;
                   self.progressLabel.text = @"Copied text to clipboard";
               }
               
           }
           else {
               UIImageWriteToSavedPhotosAlbum(testImage, nil, nil, nil);
               self.progressLabel.text = @"Wrote image to photo album";
           }
           
           return;
       }
       
       int frameNum = tgxf_get_current_frame_num();
       int total = tgxf_get_total_num_frames();
       int errors = tgxf_get_num_errors();
       int maxErrors = (int)[[NSUserDefaults standardUserDefaults] integerForKey: @"MaxErrors"];
       int fps = tgxf_get_fps();
       
       tgxf_set_max_errors(maxErrors);
       tgxf_set_max_timeout((int)[[NSUserDefaults standardUserDefaults] integerForKey: @"Timeout"]);
       
       int seconds = (total-frameNum)/fps;
       
       self.errorsLabel.text = [NSString stringWithFormat:@"Errors %d of %d (Max)", errors, maxErrors];
       self.frameLabel.text = [NSString stringWithFormat:@"Frame: %d of %d", MAX(MIN(frameNum, total), 0), total];
       self.progressLabel.text = [NSString stringWithFormat:@"%dmin %dsec to go", seconds/60, seconds%60];
       self.progressBar.progress = (float)frameNum/(float)total;
       
        break;
    }
}

- (BOOL) shouldAutorotateToInterfaceOrientation: (UIInterfaceOrientation) orient
{
    // auto-rotation is supported
    return(YES);
}

- (void) willRotateToInterfaceOrientation: (UIInterfaceOrientation) orient
                                 duration: (NSTimeInterval) duration
{
    // compensate for view rotation so camera preview is not rotated
    [self.readerView willRotateToInterfaceOrientation: orient
                                        duration: duration];
}

- (void) willAnimateRotationToInterfaceOrientation: (UIInterfaceOrientation) orient
                                          duration: (NSTimeInterval) duration
{
    // perform rotation in animation loop so camera preview does not move
    // wrt device orientation
    [self.readerView setNeedsLayout];
}

@end
