//
//  UploadViewController.m
//  TGXF
//
//  Created by James Hudson on 3/03/2014.
//  Copyright (c) 2014 James Hudson. All rights reserved.
//

#import "UploadViewController.h"
#import "FileTypeSelectorViewController.h"

#include "tgxf-transmit-pull.h"


static int MaxFileSizeBytes = 65535;

@interface UploadViewController () {
    char * dataToSend;
}
@property (strong, nonatomic) IBOutlet UIImageView *barcodeDisplayImageView;
@property (strong, nonatomic) IBOutlet UILabel *frameLabel;

@property (strong, nonatomic) IBOutlet UILabel *progressLabel;
@property int frameNumber;
@property NSTimer * showFrameTimer;
@property (strong, nonatomic) IBOutlet UIButton *startPauseToggledButton;
@property (strong, nonatomic) IBOutlet UIProgressView *progressBar;

@end

@implementation UploadViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
}

- (void) viewWillAppear:(BOOL)animated {
    [UIApplication sharedApplication].idleTimerDisabled = YES;
    

    
 /*   pull_frame(self.frameNumber, tgxf_packet, read_bytes,
               mode_ecc_level, mode_pixel_size, mode_margin_size, in_ascii);*/
}

- (void) viewDidDisappear:(BOOL)animated {
    [self stopTGXFSession];
    [UIApplication sharedApplication].idleTimerDisabled = NO;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
- (IBAction)startPauseToggled:(id)sender {
    if (self.showFrameTimer) {
        [self.showFrameTimer invalidate];
        self.showFrameTimer = nil;
        [self.startPauseToggledButton setTitle:@"Resume" forState:UIControlStateNormal];
    }
    else {
        NSUserDefaults * defaults = [NSUserDefaults standardUserDefaults];
        [self.startPauseToggledButton setTitle:@"Pause" forState:UIControlStateNormal];
        int fps = (int)[defaults integerForKey:@"FramesPerSecond"];
        float interval = 1.0f / (float)fps;
        self.showFrameTimer = [NSTimer scheduledTimerWithTimeInterval:interval target:self selector:@selector(pullNextTGXFFrame) userInfo:nil repeats:YES];
    }
}

- (void) startTGXFSession: (NSData*) toSend filename: (NSString*) filename{
    NSUserDefaults * defaults = [NSUserDefaults standardUserDefaults];
    
    unsigned int qrCodeVersion = (unsigned int)[defaults integerForKey:@"FrameResolution"];
    
    int len = (int)toSend.length;
    
    [self stopTGXFSession];
    
    dataToSend = malloc(len);
    memcpy(dataToSend, (char*)toSend.bytes, len);
    
    int fps = (int)[defaults integerForKey:@"FramesPerSecond"];
    tgxf_init_session(dataToSend, len, qrCodeVersion, [filename UTF8String], fps);
    
    self.frameNumber = 1;
    [self pullNextTGXFFrame];
}

- (void) stopTGXFSession {
    if (self.showFrameTimer) {
        [self.showFrameTimer invalidate];
        self.showFrameTimer = nil;
    }
    
    if (dataToSend != nil) {
        free(dataToSend);
        dataToSend = nil;
    }
}

- (void) pullNextTGXFFrame {
    int totalFrames = tgxf_get_num_frames();
    
    if (self.frameNumber <= totalFrames+5) { // plus control frames
        QRcode * code;
        tgxf_pull_frame(self.frameNumber, &code);
        
        char * bitmap = malloc(code->width * code->width);
        
        CGColorSpaceRef colorSpaceRGB = CGColorSpaceCreateDeviceGray();
        CGContextRef context = CGBitmapContextCreate(bitmap, code->width, code->width, 8, code->width, colorSpaceRGB, 0);

        if (context) {
            for (int i=0; i< code->width * code->width; i++) {
                bitmap[i] = (code->data[i] & 0x1) ? 0 : 255;
            }
        
            self.barcodeDisplayImageView.image = [[UIImage alloc] initWithCGImage:CGBitmapContextCreateImage(context)];
        }
        
        self.barcodeDisplayImageView.layer.magnificationFilter = kCAFilterNearest;
        
        CGContextRelease (context);
        free (bitmap);
        
        int frameNumber = MAX(MIN(self.frameNumber, totalFrames), 0);
        
        
        NSUserDefaults * defaults = [NSUserDefaults standardUserDefaults];
        int seconds = (totalFrames-self.frameNumber)/(float)[defaults integerForKey:@"FramesPerSecond"];
        self.frameLabel.text = [NSString stringWithFormat:@"Frame: %d of %d", frameNumber, totalFrames];
        self.progressLabel.text = [NSString stringWithFormat:@"%dmin %dsec to go", seconds/60, MAX(0, seconds%60)];
        self.progressBar.progress = (float)frameNumber/(float)totalFrames;
        
        self.frameNumber++;
    }
    else {
        self.startPauseToggledButton.enabled = NO;
        self.startPauseToggledButton.hidden = YES;
        [self stopTGXFSession];
    }
}

- (void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"FileTypeSelectorViewController"])
    {
        FileTypeSelectorViewController *destinationViewController = [segue destinationViewController];
        destinationViewController.delegate = self;
    }
}


-(void) userSelectedFileWithData:(NSData *)data filename:(NSString *)filename {
    
    if (data.length > MaxFileSizeBytes) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"File too large"
                                                        message:[NSString stringWithFormat: @"Your chosen file is too large. Please choose one which is under %i bytes.", MaxFileSizeBytes]
                                                       delegate:nil
                                              cancelButtonTitle:@"OK"
                                              otherButtonTitles:nil];
        [alert show];
    }
    else {
        [self startTGXFSession: data filename:filename];
        self.startPauseToggledButton.enabled = YES;
        self.startPauseToggledButton.hidden = NO;
        self.frameNumber = 1;
        [self.startPauseToggledButton setTitle:@"Begin" forState:UIControlStateNormal];
    }
}

@end
